# coding=UTF-8
'''
    This file is part of crawler-mswl.

    crawler-mswl is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    crawler-mswl is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with crawler-mswl.  If not, see <http://www.gnu.org/licenses/>.

'''

import urllib2
import logging
from BeautifulSoup import BeautifulSoup as Soup
from urllib2 import HTTPError, URLError
import traceback
from httplib import BadStatusLine

class UrlContentRetrieve:
    '''Controller class to manage url and retrieve content using BeautifulSoup.

    Created on 27/09/2012

    @author: Ricardo García Fernández
    @mail: ricardogarfe@gmail.com

    '''

    def __init__(self, top_url):
        '''Initializes UrlContentRetrieve with default user_agent.
        
        Keyword arguments:
        top_url -- URL from top web level.
        
        '''
        
        # Setup Log
        self.setup_log()

        # Define user agent
        user_agent = "Mozilla/5.0 (X11; U; Linux x86_64; en-Us) \
            AppleWebKit/534.7 (KHTML, Like Gecko) Chrome \
            /7.0.517.41 Safari/534.7"

        self.opener = urllib2.build_opener()
        self.opener.addheaders = [('User-agent', user_agent)]
        
        if top_url.endswith("/"):
            self.top_url = top_url[:-1]
        self.top_url = top_url

    def url_content(self, target_url):
        ''' Retrieve URL raw Content.
        
        Keyword arguments:
        target_url -- url to inspect.
        
        '''
        
        # Get correct format of URL
        target_url = self.generate_correct_url(target_url)

        try:
            raw_code = self.opener.open(target_url)
            soup_code = Soup(raw_code, fromEncoding="utf-8")
        except HTTPError, http_error:
            self.logger.error("HttpError with url:\t" + target_url + \
                              "\nException message:\t " + \
                                str(http_error)\
                                + "\nStack trace:\t" + \
                                traceback.format_exc())
            return None
        except URLError, url_error:
            self.logger.error("URLError with url:\t" + target_url + \
                              "\nException message:\t " + \
                                str(url_error)\
                                + "\nStack trace:\t" + \
                                traceback.format_exc())
            return None
        except BadStatusLine, bad_status:
            self.logger.error("BadStatusLine with url:\t" + target_url + \
                              "\nException message:\t " + \
                                str(bad_status)\
                                + "\nStack trace:\t" + \
                                traceback.format_exc())
            return None
        except Exception, exception:
            self.logger.error("Exception with url:\t" + target_url + \
                              "\nException message:\t " + \
                                str(exception)\
                                + "\nStack trace:\t" + \
                                traceback.format_exc())
            return None
            
        return soup_code

    def generate_correct_url(self, url_to_check):
        ''' Check the url and generate a new one well formed if is not complete.
            * Url management starts with '/' or /wiki/ - InvalidURL
            * Url management starts with '//' - HTTPError
            * Url format t0 utf-8 - http://es.wikipedia.org//yi.wikipedia.org/wiki/%D7%B0%D7%99%D7%A7%D7%99%D7%A4%D6%BC%D7%A2%D7%93%D7%99%D7%A2:%D7%91%D7%A8%D7%95%D7%9B%D7%99%D7%9D_%D7%94%D7%91%D7%90%D7%99%D7%9D
        
        Keyword arguments:
        url_to_check -- URL to check

        '''

        try:
            url_to_check = str(url_to_check.encode('utf-8'))
        except UnicodeEncodeError, unicode_error:
            self.logger.warning("UnicodeEncodeError with url:\t" + \
                              url_to_check + "\nException message:\t " + \
                                str(unicode_error)\
                                + "\nStack trace:\t" + \
                                traceback.format_exc())
            return None

        if url_to_check.startswith("//"):
            url_to_check = "http:" + url_to_check
        elif url_to_check.startswith("/"):
            url_to_check = self.top_url + url_to_check
        elif url_to_check.startswith("#"):
            url_to_check = None
        elif not url_to_check.find("https://") and \
            not url_to_check.find("http://"):
            url_to_check = self.top_url + url_to_check

        return url_to_check

    def retrieve_formatted_links(self, soup_code):
        ''' Retrieve the links in correct format into a list.
        
        '''

        formatted_links = []
        raw_links = []

        # Extract all links
        for link in soup_code.findAll('a') :
            if link.has_key('href'):
                raw_links.append(link['href'])
        
        # Convert to correct format
        for raw_link in raw_links:
            formatted_link = self.generate_correct_url(raw_link)
            if formatted_link:
                formatted_links.append(formatted_link)
            
        return formatted_links

    def setup_log(self):
        '''Setup Python logging.
        
        '''

        self.logger = logging.getLogger('UrlContentRetrieve')
        self.hdlr = logging.FileHandler('/var/tmp/crawler.log')
        self.formatter = logging.Formatter('%(asctime)s %(levelname)s \
            %(filename)s %(message)s')
        self.hdlr.setFormatter(self.formatter)
        self.logger.addHandler(self.hdlr)
        self.logger.setLevel(logging.WARNING)



# coding=UTF-8
'''
    This file is part of crawler-mswl.

    crawler-mswl is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    crawler-mswl is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with crawler-mswl.  If not, see <http://www.gnu.org/licenses/>.

'''

import unittest
from controllers.url_content_retrieve import UrlContentRetrieve
from urllib2 import URLError

class TestUrlContentRetrieve(unittest.TestCase):
    '''Test class for methods defined in UrlContentRetrieve class.
    
    Created on 01/10/2012

    @author: Ricardo García Fernández
    @mail: ricardogarfe@gmail.com

    '''

    def setUp(self):
        pass


    def tearDown(self):
        pass


    def test_generate_correct_url(self):
        ''' Test method for retrieve the correct URL
        '''
        
        starting_url = "http://localhost"
        url_content_retrieve = UrlContentRetrieve(starting_url)

        # Test 1
        url_to_check = "http://localhost"
        expected_url = "http://localhost"
        returned_url = url_content_retrieve.generate_correct_url(url_to_check)
        
        self.assertEqual(returned_url, expected_url, \
                         "Error converting URL\n:\t* from: "\
                          + url_to_check + "\n\t* to: " + expected_url + \
                          "\nReturned value: " + returned_url)

        # Test 2    
        url_to_check = "/media"
        expected_url = "http://localhost/media"
        returned_url = url_content_retrieve.generate_correct_url(url_to_check)
        
        self.assertEqual(returned_url, expected_url, \
                         "Error converting URL\n:\t* from: "\
                          + url_to_check + "\n\t* to: " + expected_url + \
                          "\nReturned value: " + returned_url)

        # Test 3
        url_to_check = "//mediateca.org"
        expected_url = "http://mediateca.org"
        returned_url = url_content_retrieve.generate_correct_url(url_to_check)
        
        self.assertEqual(returned_url, expected_url, \
                         "Error converting URL\n:\t* from: "\
                          + url_to_check + "\n\t* to: " + expected_url + \
                          "\nReturned value: " + returned_url)
        
    def test_url_content(self):
        '''Test method for retrieve content from url.
        '''
        
        # Initialize object
        starting_url = "http://non-existing-url.com"
        url_content_retrieve = UrlContentRetrieve(starting_url)

        # Test 1 - Variables
        # Return error
        target_url = "http://non-existing-url.com"
        expected_exception = \
            "<urlopen error [Errno -2] Name or service not known>"
        try:
            returned_soup_code = url_content_retrieve.url_content(target_url)
            self.assertIsNone(returned_soup_code, \
                              "Url content has return a value:\n"\
                              + str(returned_soup_code))
        except URLError, url_error:
            self.assertEqual(expected_exception, str(url_error), "Exception "\
                             "URLError is not correct with message:\n"\
                             + str(url_error))
        
        # Test 2 - enconding
        starting_url = "http://es.wikipedia.org"
        url_content_retrieve = UrlContentRetrieve(starting_url)
        target_url = \
            "http://es.wikipedia.org/wiki/Depresi%C3%B3n_tropical_Diez_(2007)"
        expected_enconding = "utf-8"
        
        returned_soup_code = url_content_retrieve.url_content(target_url)
        
        returned_enconding = returned_soup_code.originalEncoding

        self.assertEqual(expected_enconding, returned_enconding, \
                         "Encondin doesn't match:\n\t* Expected: "\
                         + expected_enconding + "\n\t* Returned: "\
                         + returned_enconding)
            

if __name__ == "__main__":
    #import sys;sys.argv = ['', 'Test.testName']
    unittest.main()



# coding=UTF-8
'''
Created on 27/09/2012

Setuptools setup file

@author: Ricardo García Fernández
@mail: ricardogarfe@gmail.com
'''
 
from setuptools import setup, find_packages

setup (name="ricardo_crawler",
        version="01.00.01",
        packages=find_packages(exclude=["test", "test.*"]),
        scripts=['ricardo_crawler.py'],
        install_requires=['BeautifulSoup'],
        package_data={'pyricardo_crawler':[''], },
        author='Ricardo García Fernández',
        author_email='ricardogarfe@gmail.com',
        description='My first Web Scrapper in Python',
        license = 'GNU GPLv3',
        keywords='web-crawler',
        url='https://github.com/ricardogarfe/crawler-mswl',
        long_description='Web Scrapper in Python for Development Tools Subject from MSWL - URJC',
        download_url='https://github.com/ricardogarfe/crawler-mswl/zipball/v01.00.01',
        )
